/*
 *    Copyright 2009-2010 The Rocoto Team
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.googlecode.rocoto.configuration.resolver;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Provider;

/**
 * 
 *
 * @author Simone Tripodi
 * @since 4.0
 * @version $Id: PropertiesResolver.java 575 2010-10-24 13:13:00Z simonetripodi $
 */
public final class PropertiesResolver implements Provider<String> {

    /**
     * The symbol that indicates a variable begin.
     */
    private static final String VAR_BEGIN = "$";

    /**
     * The symbol that separates the key name to the default value.
     */
    private static final String PIPE_SEPARATOR = "|";

    /**
     * The appenders list have to be invoked when resolving variables, in the given order.
     */
    private final List<Appender> appenders = new ArrayList<Appender>();

    /**
     * Flag to indicate the text fragment contains ${} keys.
     */
    private boolean containsKeys = false;

    /**
     * The Injector instance used to resolve variables.
     */
    @Inject
    private Injector injector;

    /**
     * Creates a new properties resolver instance.
     *
     * @param pattern the text fragment has to be parsed and extract keys.
     */
    public PropertiesResolver(final String pattern) {
        int prev = 0;
        int pos;
        while ((pos = pattern.indexOf(VAR_BEGIN, prev)) >= 0) {
            if (pos > 0) {
                this.appenders.add(new TextAppender(pattern.substring(prev, pos)));
            }
            if (pos == pattern.length() - 1) {
                this.appenders.add(new TextAppender(VAR_BEGIN));
                prev = pos + 1;
            } else if (pattern.charAt(pos + 1) != '{') {
                if (pattern.charAt(pos + 1) == '$') {
                    this.appenders.add(new TextAppender(VAR_BEGIN));
                    prev = pos + 2;
                } else {
                    this.appenders.add(new TextAppender(pattern.substring(pos, pos + 2)));
                    prev = pos + 2;
                }
            } else {
                int endName = pattern.indexOf('}', pos);
                if (endName < 0) {
                    throw new IllegalArgumentException("Syntax error in property: " + pattern);
                }
                StringTokenizer keyTokenizer = new StringTokenizer(pattern.substring(pos + 2, endName), PIPE_SEPARATOR);
                String key = keyTokenizer.nextToken().trim();
                String defaultValue = null;
                if (keyTokenizer.hasMoreTokens()) {
                    defaultValue = keyTokenizer.nextToken().trim();
                }
                this.appenders.add(new KeyAppender(key, defaultValue));
                prev = endName + 1;
                this.containsKeys = true;
            }
        }
        if (prev < pattern.length()) {
            this.appenders.add(new TextAppender(pattern.substring(prev)));
        }
    }

    /**
     * Return true, if the text contains at least one key in the ${} pattern,
     * false otherwise.
     *
     * @return true, if the text contains at least one key in the ${} pattern,
     *         false otherwise.
     */
    public boolean containsKeys() {
        return this.containsKeys;
    }

    /**
     * Set the Injector instance used to resolve variables.
     *
     * @param injector the Injector instance used to resolve variables.
     */
    public void setInjector(Injector injector) {
        this.injector = injector;
    }

    /**
     * {@inheritDoc}
     */
    public String get() {
        StringBuilder buffer = new StringBuilder();
        for (Appender appender : this.appenders) {
            appender.append(buffer, this.injector);
        }
        return buffer.toString();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        return this.appenders.toString();
    }

}
